#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import subprocess
from datetime import datetime
from yaml import load, CLoader
from jinja2 import Template

from . import conf
from . import notification


def exec_cmd(cmd):
    """Execute a command in a sub process."""
    bash_string = r"""#!/bin/bash
    set -e
    {}
    """.format(
        cmd
    )
    result = subprocess.check_output(
        bash_string, shell=True, executable="/bin/bash", text=True
    )
    return result.strip()


def run():
    """Main function responsible to trigger the tests."""
    # Creates the command line parser.
    parser = argparse.ArgumentParser(prog="guardian")

    parser.add_argument(
        "-c",
        "--config",
        dest="config_file",
        required=True,
        help="Configuration file (YAML).",
    )
    parser.add_argument(
        "--email",
        dest="email_notification",
        action="store_true",
        help="Send notification of failed test(s) via email.",
    )
    parser.add_argument(
        "--irc",
        dest="irc_notification",
        action="store_true",
        help="Send notification of failed test(s) via IRC.",
    )
    parser.add_argument(
        "--html",
        dest="html_report",
        action="store_true",
        help="Generate a HTML status page.",
    )

    parser.set_defaults(email_notification=False, html_report=False)

    arguments = parser.parse_args()

    # Read the configuration file
    with open(arguments.config_file, "r") as f:
        stream = f.read()
        data = load(stream, Loader=CLoader)

    # Pass through all the tests.
    results = []
    all_errors = []
    reports = []
    services = data["services"]
    start_date = datetime.now()
    for service in services:
        report = {
            "service_name": service,
            "is_ok": True,
            "errors": [],
            "nb_test": len(services[service]["tests"]),
        }
        print("+ " + service)
        for test in services[service]["tests"]:
            print(" - " + test)
            if services[service]["tests"][test].get("disabled", False):
                # test disabled in the configuration file, skip it
                print("     Test disabled in the configuration file.")
                continue
            cmd = services[service]["tests"][test]["probe"]
            expected = services[service]["tests"][test]["expected"]
            # print("   - " + cmd)
            # print("   - " + expected)
            result = exec_cmd(cmd)
            if result == expected:
                results.append(True)
                print("     ✅")
            else:
                results.append(False)
                all_errors.append((service, test))
                report["is_ok"] = False
                report["errors"].append(test)
                print("     ❌")

        reports.append(report)

    end_date = datetime.now()
    print("Execution time: {} seconds.".format((end_date - start_date).total_seconds()))

    if all(results):
        print("✨ 🌟 ✨ All {} tests are successful.".format(len(results)))
    else:
        print(
            "{number} error{plural} occurred.".format(
                number=len(all_errors), plural="s" if len(all_errors) > 1 else ""
            )
        )
        if arguments.email_notification:
            print("Sending email notification...")
            # TODO

        if arguments.irc_notification:
            print("Sending IRC notification...")
            for error in all_errors:
                notification.irker(
                    "Error for service '{}'. Test: {}".format(error[0], error[1])
                )

    if arguments.html_report:
        print("Generating HTML status page...")
        template = Template(open("guardian/templates/report.html").read())
        outputHTML = template.render(
            reports=reports, end_date=end_date, time=end_date - start_date
        )
        with open("reports/index.html", "w") as f:
            f.write(outputHTML)


if __name__ == "__main__":
    # Point of entry in execution mode.
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import configparser

# load the configuration
config = configparser.SafeConfigParser()
try:
    config.read("guardian/config/conf.cfg")
except:
    config.read("guardian/config/conf.cfg.sample")


IRC_CHANNEL = config.get("irc", "channel")
IRKER_HOST = config.get("irc", "host")
IRKER_PORT = int(config.get("irc", "port"))

MAIL_FROM = config.get("email", "mail_from")
MAIL_TO = [config.get("email", "mail_to")]
SMTP_SERVER = config.get("email", "smtp")
USERNAME = config.get("email", "username")
PASSWORD = config.get("email", "password")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['guardian', 'guardian.probes']

package_data = \
{'': ['*'],
 'guardian': ['config/conf.cfg.sample',
              'config/conf.cfg.sample',
              'config/services.yaml.example',
              'config/services.yaml.example',
              'templates/*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0', 'PyYAML>=5.4.1,<6.0.0']

entry_points = \
{'console_scripts': ['guardian = guardian.main:run']}

setup_kwargs = {
    'name': 'guardian',
    'version': '0.2.0',
    'description': 'Monitor the status of a set of services.',
    'long_description': '# Guardian\n\nMonitor the status of a set of services. Characteristics:\n\n- definition of the services to monitor with a YAML file;\n- tests performed by custom scripts (Shell scripts, Python scripts, etc.);\n- no database and serverless;\n- generation of HTML status page;\n- email notifications;\n- IRC notifications.\n\n\n## Usage\n\n```bash\n$ guardian --help\nusage: guardian [-h] -c CONFIG_FILE [--email] [--irc] [--html]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -c CONFIG_FILE, --config CONFIG_FILE\n                        Configuration file (YAML).\n  --email               Send notification of failed test(s) via email.\n  --irc                 Send notification of failed test(s) via IRC.\n  --html                Generate a HTML status page.\n```\n\n\nIn order to use notification via IRC you need to install\n[irker](http://www.catb.org/~esr/irker/). irker is very easy to install and\nto run, no configuration is needed. Once executed, irker will wait for JSON\nformatted messages on the port 6659. irker will automatically join the channel\nyou have specified in the\n[Guardian configuration file](guardian/config/conf.cfg.sample#L2).\nirker will maintain connection state for multiple channels, avoiding obnoxious\njoin/leave spam.\n\nConfigurations related to the sending of emails are in the\n[same file](guardian/config/conf.cfg.sample#L5).\n\n\n## Examples\n\nThe goal of the INI configuration file is to set global variables (IRC channel, SMTP\nserver, etc.). If you do not create your own configuration file, the default one will\nbe used automatically.\n\nThe services to monitor must be described in one (or several) YAML file(s).\n\n\n```bash\n$ cp guardian/config/config.cfg.sample guardian/config/config.cfg\n$ cp guardian/config/services.yaml.example guardian/config/monarc-services.yaml\n\n\n$ guardian -c guardian/config/monarc-services.yaml\n+ Service my.monarc.lu\n - Test HTTPS front server\n     ✅\n - Test HTTPS FO1 (casesmodels)\n     ✅\n - Test HTTPS FO2 (casesmodels2)\n     ✅\n - Test HTTPS Back Office\n     ✅\n+ Service MOSP\n - Test API\n     ✅\n+ Stats Service\n - Test /about.json endpoint\n     ✅\n✨ 🌟 ✨ All 6 tests are successful.\n```\n\n\nWith email notification:\n\n```bash\n$ guardian -c guardian/config/google-services.yaml --email\n+ Google services\n - Test GMail\n     ✅\n - Test Web search\n     ❌\n - Test Google Drive\n     ✅\n1 error occurred.\nExecution time: 0:00:00.793011\nSending email notification...\n```\n\nYou can combine email notifications, IRC notifications and HTML reporting.\n\n\n## Contributing\n\nPatches and questions? Send to my [public\ninbox](https://lists.sr.ht/~cedric/public-inbox):\n[`~cedric/public-inbox@lists.sr.ht`](mailto:~cedric/public-inbox@lists.sr.ht).\nThanks!\n\n\n## License\n\n[Guardian](https://sr.ht/~cedric/guardian) is licensed under\n[GNU Affero General Public License version 3](https://www.gnu.org/licenses/agpl-3.0.html).\n\nCopyright (C) 2021 [Cédric Bonhomme](https://www.cedricbonhomme.org)\n',
    'author': 'Cédric Bonhomme',
    'author_email': 'cedric@cedricbonhomme.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://sr.ht/~cedric/guardian',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import socket
import smtplib
from email.mime.text import MIMEText

from guardian import conf


def irker(message):
    """Send a JSON formatted message to an instance of irker."""
    data = {"to": conf.IRC_CHANNEL, "privmsg": message}
    try:
        s = socket.create_connection((conf.IRKER_HOST, conf.IRKER_PORT))
        s.sendall(json.dumps(data).encode("utf-8"))
    except socket.error as e:
        sys.stderr.write("irkerd: write to server failed: %r\n" % e)


def mail(mfrom, mto, message):
    """Send the notification via mail."""
    email = MIMEText(message, "plain", "utf-8")
    email["From"] = mfrom
    email["To"] = mto
    email["Subject"] = "Guardian : Alert"
    # email['Text'] = message

    server = smtplib.SMTP(conf.SMTP_SERVER)
    server.login(conf.USERNAME, conf.PASSWORD)
    server.send_message(email)
    server.quit()
